           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           %%%    Hybrid Simulation Current Assessment Measures    %%%
           %%%                                                     %%%
           %%%     Prepared by:                                    %%%
           %%%     CARLOS ANDRES RIASCOS GONZALEZ, Ge(Gaby) Ou,    %%%
           %%%     and Shirley Dyke                                %%%
           %%%                                                     %%%
           %%%    Published on: 10/15/2014                         %%% 
           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
classdef cam
    
    properties
        
        name; eDM; eMD; TI; A; d; Enrms; FEI; Test; Eerr_exp; HSEM; parameters;
       
    end
    
    methods
       
        function obj=cam(varargin)
            % obj=cam(xc,xm,Fm,Fs,M,C,K,Eemax,l,method,name) is a function that evaulates
            % some Current Assessment Measures (CAM) for Hybrid Simulation Tests.
            % This function evaluates the following parameters:
            % 
            % 1) Peak Error (e)
            % -  eDM: error between the peak target displacement and the peak measured displacement
            % -  eMD: maximum difference error between target and measured
            %         displacements.
            % 2) Tracking Indicator (TI)
            % 3) Normalized Root Mean Square Error (Enrms)
            %    - Time domain
            %    - Frequency domain
            % 4) Frequecy Evaluation Index (FEI)
            % 5) Test
            % 6) Eerr_Exp
            % 7) Hybrid Simulation Error Monitoring (HSEM)
            % 
            % where:
            %
            % --- TIME DOMAIN SIGNALS ---
            %
            % xc: signal commanded in m.
            % xm: signal measured in m.
            % Fm: force measured in N.
            % Fs: sample rate in Hz.
            %
            %--- FREQUENCY DOMAIN SIGNALS ---
            %
            % yc: fft of the signal commanded (xc) 
            % ym: fft of the signal commanded (xm) 
            % ycn=|fft(yc)|^l;
            % ymn=|fft(ym)|^l;
            % l=2 (default): integer power (Guo et al., 2014)
            %
            % --- DYNAMICS PROPERTIES ---
            %
            % M: mass matrix of numerical substructure
            % C: damping matrix of numerical substructure
            % K: stiffness matrix of numerical substructure
            %
            % --- METHOD ---
            %
            % method: for numerical integration can be selected 
            %         [{'area'}|'cumtrapz'|'cycle']
            %
            % --- CONSTRAINTS --- 
            %
            % Eemax: maximum experimental strain energy 
            %
            %
            % by: Carlos Riascos (September 5, 2014)
            
                                   %%%%%%%%%%%%%%%%%%%%                             
                                   %%% INITIALIZING %%%
                                   %%%%%%%%%%%%%%%%%%%%
                                   
 global ind                                  
[xc,xm,Fm,Fs,M,C,K,Eemax,l,method,name]=iniCAM(varargin); obj.name=name;

if ind==0, return, end
            
                    %%%%%%%%%%%%%%%%%%%
                    %%%%% SIGNALS %%%%%
                    %%%%%%%%%%%%%%%%%%%
                            
                      
            %%%%%%%%%%%%%%%%%%%
            %%% Time Domain %%%
            %%%%%%%%%%%%%%%%%%%
            
            if size(xm,2)>size(xm,1),xm=xm'; end
            if size(xc,2)>size(xc,1),xc=xc'; end        
            if size(Fm,2)>size(Fm,1),Fm=Fm';end 
            if size(Eemax,2)>size(Eemax,1),Eemax=Eemax';end 
            
            L=length(xc); t=[0:L-1]'/Fs; % time vector
            
            p.xc=xc; p.xm=xm; p.fm=Fm;
            
            %%%%%%%%%%%%%%%%%%%%%%%%
            %%% Frequency Domain %%%
            %%%%%%%%%%%%%%%%%%%%%%%%
                        
            yc=fft(xc); yc=yc(1:round(end/2),:); ycn=abs(yc).^l; % input signal
            ym=fft(xm); ym=ym(1:round(end/2),:); ymn=abs(ym).^l; % output signal
            f=linspace(0,Fs/2,length(yc))'; % frequency vector
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% Dynamics properties %%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            p.M=M; p.C=C; p.K=K; 
            
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            %%%%% CURRENT ASSESSMENT MEASURES %%%%%
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            

            %%%%%%%%%%%%%%%%%%%%%%%%%
            %%% 1) Peak Error (e) %%%
            %%%%%%%%%%%%%%%%%%%%%%%%%
            
            obj.eDM=abs( (max(abs(xc))-max(abs(xm)))./max(xm) );
            obj.eMD=max(abs( xc-xm))./max(xm) ;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% 2) Tracking Indicatior (TI) %%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            A=zeros(1,size(xc,2));
            TA=zeros(1,size(xc,2));
            
            for i=1:length(xc)-1    
                A(i+1,:) = A(i,:)  + (xc(i+1,:)+xc(i,:)).*(xm(i+1,:)-xm(i,:))/2;
                TA(i+1,:)= TA(i,:) + (xm(i+1,:)+xm(i,:)).*(xc(i+1,:)-xc(i,:))/2;
            end         
            
            p.TI=(A-TA)/2;
            obj.TI=p.TI(end,:);
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% 3) Normalized Root Mean Square Error (Enrms) %%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            obj.Enrms=sqrt(sum((xc-xm).^2)./sum(xc.^2));
            p.Enrms_f=sqrt(sum(abs(yc-ym).^2))./sqrt(sum(abs(yc).^2));
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% 4) Frequecy Evaluation Index (FEI) %%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        
            obj.FEI=sum( (ym./yc).* ycn )./sum(ycn); 
            obj.A=abs(obj.FEI);
            p.phi=atan(imag(obj.FEI)./real(obj.FEI));
            p.feq=sum(ycn.*(f*ones(1,size(xc,2))))./sum(ycn); 
            obj.d=-p.phi./(2*pi*p.feq);
            
            %%%%%%%%%%%%%%%
            %%% 5) Test %%%
            %%%%%%%%%%%%%%%
            
            for i=1:size(xc,2)
            [p.xcXxm(:,i),p.lags(:,i)]=xcorr(xm(:,i),xc(:,i));  
            end
            
            [~,p.CCI]=max(p.xcXxm);
            obj.Test=diag(p.lags(p.CCI,:))';
            
            %%%%%%%%%%%%%%%%%%%
            %%% 6) Eerr_Exp %%%
            %%%%%%%%%%%%%%%%%%%
            
            p.EEc=cam_integral(t,xc,Fm,method); % Energy dissipated in the Analitical substructure 
            p.EE=cam_integral(t,xm,Fm,method); % Experimental Energy    
            p.Eerr_exp=p.EEc-p.EE;
            obj.Eerr_exp=p.Eerr_exp(end,:);
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% 7) Energy Error Indicator (EEI) %%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            dxc=diff(xc)*Fs; dxc=[dxc; dxc(end,:)]; % constant velocity at last step
            p.ED=cam_integral(t,xc,dxc*C,method); % Energy Dissipated through viscous damping in NS 
            p.ES=cam_integral(t,xc,xc*K,method); % Strain Energy
            p.EK=cumsum((dxc.^2)*M)/2; % Kinetic Energy
            
            p.Eemax=Eemax; % Strain Energy in the Hybrid Model
          
            p.EI=p.EK+p.ED+p.ES+p.EEc; % Input Energy
            [~,col]=size(p.EI);
            if length(p.Eemax)==1 && col>1, p.Eemax=p.Eemax*ones(col,1); end
            p.HSEM=p.Eerr_exp./(p.EI+ones(length(p.EI),1)*p.Eemax');
            obj.HSEM=p.HSEM(end,:);
            
            p.Fs=Fs;
            p.t=t;
            obj.parameters=p;          
                                 
function [xc,xm,Fm,Fs,M,C,K,Eemax,l,method,name] = iniCAM(P)
   
    ind=1; len=length(P);
          
        if length(P)>2 && length(P)<12

dof=min(size(P(1)));
Q={P(1:3),1,eye(dof),eye(dof),eye(dof),1,2,'area','CAM'};
I=find(isempty(P)); P(I)=Q(I);
xc=P{1}; xm=P{2}; Fm=P{3}; Fs=P{4}; M=P{5}; C=P{6}; K=P{7}; Eemax=P{8}; l=P{9}; method=P{10}; name=P{11};

        else
    
P=cell(1,11);
xc=P{1}; dof=min(size(xc)); xm=P{2}; Fm=P{3}; Fs=P{4}; M=P{5}; C=P{6}; K=P{7}; Eemax=P{8}; l=P{9}; method=P{10}; name=P{11};
ind=0;

if len<11
    errordlg('Not enough input arguments.','File Error');
elseif len>11
     errordlg('Too much input arguments.','File Error');  
else
      errordlg('Incorrect input arguments.','File Error');     
end
        end

end

        end



        
    end   

end